package de.mcs.tools.sps.holtec;

import static org.junit.Assert.fail;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.SPSCommand;
import de.mcs.tools.sps.SPSCommandData;
import de.mcs.tools.sps.SPSEmulator;
import de.mcs.tools.sps.exceptions.WrongProgramSizeException;

class TestHoltecEmualtor {

  private SPSEmulator holtectEmulator;

  @BeforeEach
  void setUp() throws Exception {
    holtectEmulator = new HoltecEmulator();
  }

  @Test
  void testOutputAllCommands() {
    System.out.println("command list holtect");
    List<SPSCommand> commands = holtectEmulator.getCommands();
    commands.forEach(c -> {
      List<SPSCommandData> commandDatas = c.getCommandDatas();
      System.out.printf("Command %s(%s):", c.getMnemonic(), c.getName());
      commandDatas.forEach(cd -> {
        System.out.printf("%s(%s) ,", cd.getMnemonic(), cd.getName());
      });
      System.out.println();
    });
  }

  @Test
  void testProgramSize() throws WrongProgramSizeException {
    assertEquals(128, holtectEmulator.getProgramSize());
    byte[] prgMemory = new byte[128];
    holtectEmulator.loadProgram(prgMemory);
  }

  @Test
  void testWrongProgramSize() throws WrongProgramSizeException {
    byte[] prgMemory = new byte[129];
    Throwable exception = assertThrows(WrongProgramSizeException.class, () -> holtectEmulator.loadProgram(prgMemory));
  }

  @Test
  void testCommandArray() {
    List<SPSCommand> commands = holtectEmulator.getCommands();
    assertEquals(14, commands.size());
    for (SPSCommand spsCommand : commands) {
      switch (spsCommand.getCommandByte()) {
      case 0x10:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x20:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x30:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x40:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x50:
        assertEquals(9, spsCommand.getCommandDatas().size());
        break;
      case 0x60:
        assertEquals(10, spsCommand.getCommandDatas().size());
        break;
      case 0x70:
        assertEquals(10, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0x80:
        assertEquals(8, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0x90:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xA0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xB0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xC0:
        assertEquals(15, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xD0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xE0:
        assertEquals(1, spsCommand.getCommandDatas().size());
        break;
      default:
        fail(String.format("unknow command %s", spsCommand.getName()));
        break;
      }
    }
  }
}
